@@ -5,8 +5,8 @@
 #define STATE_STOP_DIAL 2
 #define STATE_KILL_DIAL 3
 #define STATE_START_LINK 4
-#define STATE_STOP_PPPD 5
-#define STATE_KILL_PPPD 6
+#define STATE_STOP_LINK 5
+#define STATE_KILL_LINK 6
 #define STATE_UP 7
 #define STATE_DISCONNECT 8
 #define STATE_CLOSE 9
