@@ -43,6 +43,9 @@
 
 #include "diald.h"
 
+static char *current_dev = 0;
+static int rotate_offset = 0;
+
 /* local variables */
 
 static struct termios inittermios;      /* Initial TTY termios */
@@ -238,31 +241,48 @@ void open_modem()
     modem_hup = 0;
     modem_fd = -1;
 
-    for (i = 0; i < device_count; i++) {
-        /*
-         * March down the device list till we manage to open one up or
-  	 * we run out of devices to try.
-         */
-
-        if (lock_dev && lock(devices[i]) < 0)
-	    continue;
-
-	/* OK. Locked one, try to open it */
-	if ((modem_fd = open(devices[i], O_RDWR | O_NDELAY)) >= 0)
-	    break;
-	else {
-	   syslog(LOG_ERR,"Error opening device %s: %m",devices[i]);
+    if (req_pid) {
+	/* The user has specified a device. Use it, no search or lock needed. */
+	if ((modem_fd = open(req_dev, O_RDWR | O_NDELAY)) < 0) {
+	    syslog(LOG_ERR,"Can't open requested device '%s'",req_dev);
+	    killpg(req_pid,SIGKILL);
+	    kill(req_pid,SIGKILL);
+	    req_pid = 0;
+	    dial_status = -1;
+	    return;
 	}
+	current_dev = req_dev;
+    } else {
+	for (i = 0; i < device_count; i++) {
+	    current_dev = devices[(i+rotate_offset)%device_count];
+	    /*
+	     * March down the device list till we manage to open one up or
+	     * we run out of devices to try.
+	     */
 
-	/* That didn't work, get rid of the lock */
-        if (lock_dev) unlock();
+	    if (lock_dev && lock(devices[i]) < 0)
+		continue;
+
+	    /* OK. Locked one, try to open it */
+	    if ((modem_fd = open(devices[i], O_RDWR | O_NDELAY)) >= 0)
+		break;
+	    else {
+	       syslog(LOG_ERR,"Error opening device %s: %m",devices[i]);
+	    }
+	    current_dev = 0;
+
+	    /* That didn't work, get rid of the lock */
+	    if (lock_dev) unlock();
+	}
+	if (modem_fd < 0) {
+	    syslog(LOG_INFO,"Couldn't find a free device to call out on.");
+	    dial_status = -1;
+	    return;
+	}
     }
 
-    if (modem_fd < 0) {
-	syslog(LOG_INFO,"Couldn't find a free device to call out on.");
-	dial_status = -1;
-	return;
-    }
+    if (rotate_devices)
+    	rotate_offset = (rotate_offset+1)%device_count;
 
     /* Make sure we are the session leader */
     if ((npgrpid = setsid()) >= 0)
@@ -292,24 +312,68 @@ void open_modem()
     /* So we don't try to restore if we die before this */
     restore_term = 1;
 
-    /* Clear the NDELAY flag now (line is in CLOCAL) */
+    /* Clear the NDELAY flag now */
     if (fcntl(modem_fd,F_SETFL,fcntl(modem_fd,F_GETFL)&~(O_NDELAY)) < 0)
 	syslog(LOG_ERR, "failed to clear O_NDELAY flag: %m"), die(1);
 
-    /* hang up and then start again */
-    set_up_tty(modem_fd, 1, 0);
+    if (!req_pid) {
+	/* hang up and then start again */
+	set_up_tty(modem_fd, 1, 0);
+	sleep(1);
+	set_up_tty(modem_fd, 1, inspeed);
+
+	/* Get rid of any initial line noise after the hangup */
+	tcflush(modem_fd, TCIOFLUSH);
+	fork_dialer(connector, modem_fd, modem_fd);
+    } else {
+	/* someone else opened the line, we just set the mode */
+	set_up_tty(modem_fd, 0, inspeed);
+    }
+}
+
+/*
+ * Reopen up a modem that closed on a sighup and set up the desired parameters.
+ */
+void reopen_modem()
+{
+    int npgrpid;
+
+    if(debug&DEBUG_VERBOSE)
+	syslog(LOG_INFO,"Reopening modem device");
+    close(modem_fd);
     sleep(1);
-    set_up_tty(modem_fd, 1, inspeed);
+    if ((modem_fd = open(current_dev, O_RDWR | O_NDELAY)) < 0) {
+	syslog(LOG_ERR,"Can't reopen device '%s'",current_dev);
+    } else {
+	/* Make sure we are the session leader */
+	if ((npgrpid = setsid()) >= 0)
+	    pgrpid = npgrpid;
 
-    /* Get rid of any initial line noise after the hangup */
-    tcflush(modem_fd, TCIOFLUSH);
+	/* set device to be controlling tty */
+	/* This should only happen in SLIP mode */
+	if (mode == MODE_SLIP) {
+	    if (ioctl(modem_fd, TIOCSCTTY, 1) < 0) {
+		syslog(LOG_ERR, "failed to set modem to controlling tty: %m");
+		die(1);
+	    }
 
-    fork_dialer(connector, modem_fd, modem_fd);
+	    if (tcsetpgrp(modem_fd, pgrpid) < 0) {
+		syslog(LOG_ERR, "failed to set process group: %m");
+		die(1);
+	    }
+	}
+
+	set_up_tty(modem_fd, 1, inspeed);
+	/* Clear the NDELAY flag now */
+	if (fcntl(modem_fd,F_SETFL,fcntl(modem_fd,F_GETFL)&~(O_NDELAY)) < 0)
+	    syslog(LOG_ERR, "failed to clear O_NDELAY flag: %m"), die(1);
+    }
 }
 
 void finish_dial()
 {
-    set_up_tty(modem_fd, 0, inspeed);
+    if (!req_pid)
+        set_up_tty(modem_fd, 0, inspeed);
 }
 
 /*
@@ -332,10 +396,7 @@ void close_modem()
      */
 
     if (restore_term) {
-	if (tcsetattr(modem_fd, TCSANOW, &inittermios) < 0) {
-	    syslog(LOG_ERR,
-		"failed to restore initial modem terminal settings: %m");
-	}
+	tcsetattr(modem_fd, TCSANOW, &inittermios);
     }
 
     /*
@@ -349,6 +410,12 @@ void close_modem()
     sleep(1);
 
     close(modem_fd);
-    if (lock_dev) unlock();
+    if (req_pid) {
+	if (debug&DEBUG_VERBOSE)
+	    syslog(LOG_INFO, "Killing requesting shell pid %d",req_pid);
+	killpg(req_pid, SIGKILL);
+	kill(req_pid, SIGKILL);
+	req_pid = 0;
+    } else if (lock_dev) unlock();
     modem_fd = -1;
 }
